@@ -10,20 +10,27 @@
 #ifndef __LIBZIA_H
 #define __LIBZIA_H
 
+#ifdef _MSC_VER
+#pragma warning(error:4013)
+#endif
+
 #include <eprintf.h>
 #include <zandroid.h>
 #include <zasyncdns.h>
+#include <zavgfilter.h>
 #include <zbat.h>
 #include <zbfd.h>
 #include <zbinbuf.h>
 #include <zcall.h>
 #include <zchart.h>
 #include <zclip.h>
+#include <zcor.h>
 #include <zdebug.h>
+#include <zdht11.h>
 #include <zdir.h>
 #include <zdump.h>
 #include <zerror.h>
-#include <zfence.h>
+//#include <zfence.h>
 #include <zfhs.h>
 #include <zfiledlg.h>
 #include <zfile.h>
@@ -31,12 +38,16 @@
 #include <zgetopt.h>
 #include <zghash.h>
 #include <zglib.h>
+#include <zgpio.h>
 #include <zgptrarray.h>
 #include <zhash.h>
 #include <zhdkeyb.h>
 #include <zhttp.h>
+#include <zhttpd.h>
 #include <zia.h>
+#include <ziconv.h>
 #include <zifaces.h>
+#include <zinput.h>
 #include <zjson.h>
 #include <zlist.h>
 #include <zloc.h>
@@ -57,4 +68,12 @@
 #include <ztime.h>
 #include <zver.h>
 
+#ifdef Z_UNIX
+#include <zbus.h>
+#include <zi2c.h>
+#include <zspidev.h>
+#include <zspigpio.h>
+#include <zssd1306.h>
+#endif
+
 #endif
