@@ -21,7 +21,8 @@ void cfmakeraw (struct termios *);
 #endif
 
 #ifdef Z_MSC_MINGW
-#include <Windows.h>
+#include <windows.h>
+#include <process.h>
 #endif
 
 
@@ -66,5 +67,18 @@ char *zreg_getstr(HKEY hSecKey, const char *section, const char *entry);
 
 int z_browser(char *url);
 
+#define Z_CLR 0
+#define Z_SET 1
+#define Z_XOR 2
+
+
+#if _MSC_VER < 1700 /* 2012 */
+double round(double number);
+#endif
+
+#ifdef Z_MSC_MINGW
+#define getpid _getpid
+#endif
+
 
 #endif
