@@ -11,8 +11,10 @@
 #ifndef __ZRC_H
 #define __ZRC_H
  
+
 #include <glib.h>
-    
+
+#include <stdio.h>
 
 struct zrc{
     GHashTable *items;
@@ -27,6 +29,7 @@ void zrc_info(void);
 
 char *zrc_str(char *key, char *def);
 int zrc_int(char *key, int def);
+int zrc_double(char *key, double def); // current locales
 
 
 char *zrc_strf(char *def, char *fmt, ...);
@@ -36,4 +39,12 @@ void zrc_declare_array(char *key);
 GPtrArray *zrc_array(char *key);
 
 
+
+extern char zrc_errstr[100];
+void zrc_write_str(FILE *f, char *key, char *value);
+void zrc_write_int(FILE *f, char *key, int value);
+void zrc_write_double(FILE *f, char *key, int value, int places);
+int zrc_save(char *file, void zrc_app_write(FILE *f));
+
+    
 #endif
