@@ -22,7 +22,7 @@ struct zbinbuf *zbinbuf_init(void){
     struct zbinbuf *zbb;
 
     zbb = g_new0(struct zbinbuf, 1);
-    zbb->size = 1;//TODO 256;
+    zbb->size = 256;
     zbb->buf = g_new0(char, zbb->size);
 	zbb->buf[0] = '\0';
     zbb->len = 0;
@@ -32,11 +32,13 @@ struct zbinbuf *zbinbuf_init(void){
 
 
 void zbinbuf_free(struct zbinbuf *zbb){
+    if (zbb == NULL) return;
+
     g_free(zbb->buf);
     g_free(zbb);
 }
 
-void zbinbuf_append_bin(struct zbinbuf *zbb, void *data, int len){
+void zbinbuf_append_bin(struct zbinbuf *zbb, const void *data, int len){
     int size;
 
     if (len <= 0) return;
@@ -55,6 +57,22 @@ void zbinbuf_append(struct zbinbuf *zbb, char *str){
 	zbinbuf_append_bin(zbb, str, strlen(str));
 }
 
+int zbinbuf_sprintf(struct zbinbuf *zbb, char *fmt, ...)
+{
+	gchar *c;
+	int len;
+
+	va_list l;
+	va_start(l, fmt);
+	c = g_strdup_vprintf(fmt, l);
+	len = strlen(c);
+	zbinbuf_truncate(zbb, 0);
+	zbinbuf_append_bin(zbb, c, len);
+	g_free(c);
+	va_end(l);
+	return len;
+}
+
 int zbinbuf_sprintfa(struct zbinbuf *zbb, char *fmt, ...)
 {
 	gchar *c;
@@ -106,7 +124,13 @@ void zbinbuf_truncate(struct zbinbuf *zbb, int len){
 
 
 void zbinbuf_getstr(struct zbinbuf *zbb, int pos, char *str, int maxlen){
-	int len = zbb->len - pos + 1;
+	int len;
+
+	if (pos < 0){
+		*str = '\0';
+		return;
+	}
+	len = zbb->len - pos + 1;
 	if (len <= 0) return;
 	if (maxlen >= 0 && len > maxlen) len = maxlen;
 	g_strlcpy(str, zbb->buf + pos, len);
