@@ -126,7 +126,7 @@ gpointer zhdkeyb_main(gpointer xxx){
 
     ret = ftdi_usb_open(zhdkeyb->ftdi, ftdi_vid, ftdi_pid);
     if (ret){
-        zselect_msg_send(zhdkeyb->zsel, "HD;!;Can't open ftdi device %04x:%04x, error=%d %s", ftdi_vid, ftdi_pid, ret, ftdi_get_error_string(zhdkeyb->ftdi));
+        //zselect_msg_send(zhdkeyb->zsel, "HD;!;Can't open ftdi device %04x:%04x, error=%d %s", ftdi_vid, ftdi_pid, ret, ftdi_get_error_string(zhdkeyb->ftdi));
         dbg("HD;!;Can't open ftdi device %04x:%04x, error=%d %s", ftdi_vid, ftdi_pid, ret, ftdi_get_error_string(zhdkeyb->ftdi));
         if (ret==-8) zselect_msg_send(zhdkeyb->zsel, "HD;!;Maybe run program as root");
         return NULL;
