@@ -1,6 +1,6 @@
 /*
     zifaces.c - Network interfaces detection
-    Copyright (C) 2002-2012  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2020  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of samba www.samba.org
 
     This program is free software; you can redistribute it and/or
@@ -44,6 +44,10 @@
 */
 
 #include <libziaint.h>
+#include <zstr.h>
+
+#define MAX_INTERFACES 128
+
 #ifndef Z_MSC_MINGW
 
 
@@ -54,16 +58,22 @@
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <sys/socket.h>
+#ifdef Z_HAVE_LINUX_WIRELESS_H
+#include <linux/wireless.h>
+#endif
+#include <glib.h>
 
 #include <zifaces.h>
 
+#include <zdebug.h>
+
 
 /* this works for Linux 2.2, Solaris 2.5, SunOS4, HPUX 10.20, OSF1
    V4.0, Ultrix 4.4, SCO Unix 3.2, IRIX 6.4, FreeBSD 3.2. and cygwin
 
    It probably also works on any BSD style system.  */
 
-static int _interfaces_get(struct ziface_struct *ifaces, int max_interfaces)
+static int _interfaces_get(struct ziface_struct *ifaces, int max_interfaces, int up_only)
 {  
     struct ifconf ifc;
     char buff[8192];
@@ -99,6 +109,11 @@ static int _interfaces_get(struct ziface_struct *ifaces, int max_interfaces)
         iname = ifr[i].ifr_name;
         ipaddr = (*(struct sockaddr_in *)&ifr[i].ifr_addr).sin_addr;
 
+        memset(ifaces[total].mac, 0, 6);
+        if (ioctl(fd, SIOCGIFHWADDR, &ifr[i]) == 0) {
+            memcpy(ifaces[total].mac, ifr[i].ifr_hwaddr.sa_data, 6);
+        }
+
         if (ioctl(fd, SIOCGIFFLAGS, &ifr[i]) != 0) {
             continue;
         }  
@@ -109,7 +124,7 @@ static int _interfaces_get(struct ziface_struct *ifaces, int max_interfaces)
 
         if (ioctl(fd, SIOCGIFNETMASK, &ifr[i]) != 0) {
             continue;
-        }  
+        }        
 
         nmask = ((struct sockaddr_in *)&ifr[i].ifr_addr)->sin_addr;
 
@@ -125,44 +140,82 @@ static int _interfaces_get(struct ziface_struct *ifaces, int max_interfaces)
     return total;
 }  
 
-static int iface_comp(struct ziface_struct *i1, struct ziface_struct *i2)
-{
-    int r;
-    r = strcmp(i1->name, i2->name);
-    if (r) return r;
-    r = ntohl(i1->ip.s_addr) - ntohl(i2->ip.s_addr);
-    if (r) return r;
-    r = ntohl(i1->netmask.s_addr) - ntohl(i2->netmask.s_addr);
-    return r;
+
+
+
+
+int ziface_wifi_stats(struct zwifi_stat *stat, char *devname, int clear){
+#ifdef Z_HAVE_LINUX_WIRELESS_H
+	int sockfd;
+	struct iw_statistics stats;
+	struct ziface_struct ifaces[MAX_INTERFACES];
+
+	int total = zifaces_get(ifaces, MAX_INTERFACES, 1);
+	int i;
+
+	if (total <= 0) return -1;
+
+	for (i = 0; i < total; i++){
+		int b;
+		struct in_addr iaddr;
+
+		if (devname != NULL){
+			if (strcmp(devname, ifaces[i].name) != 0) continue;
+		}
+
+
+		struct iwreq req = {
+			.u.data = {
+				.pointer = &stats,
+				.length = sizeof(struct iw_statistics),
+				.flags = clear
+			}
+		};
+		//printf("%-10s ", ifaces[i].name);
+		g_strlcpy(req.ifr_name, ifaces[i].name, IFNAMSIZ);
+
+		/* Any old socket will do, and a datagram socket is pretty cheap */
+		if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
+			continue;
+		}
+
+		/* Perform the ioctl */
+		if (ioctl(sockfd, SIOCGIWSTATS, &req) == -1) {
+			close(sockfd);
+			continue;
+		}
+
+		stat->raw_level = stats.qual.level;
+		stat->raw_noise = stats.qual.noise;
+		stat->raw_qual = stats.qual.qual;
+		stat->raw_updated = stats.qual.updated & IW_QUAL_LEVEL_UPDATED;
+
+
+
+		struct iw_range range;
+
+		req.u.data.pointer = (caddr_t)&range;
+		req.u.data.length = sizeof(struct iw_range);
+		req.u.data.flags = 0;
+
+		if (ioctl(sockfd, SIOCGIWRANGE, &req) < 0){
+			close(sockfd);
+			continue;
+		}
+
+		stat->raw_max_qual = range.max_qual.qual;
+
+		close(sockfd);
+
+		stat->quality_percent = (stat->raw_qual * 100) / stat->raw_max_qual;
+		stat->level_dbm = range.sensitivity - 256 + (unsigned char)stats.qual.level;
+
+		return 0;
+	}
+#endif
+	return -1;
 }
 
-/* this wrapper is used to remove duplicates from the interface list generated
-   above */
-int zifaces_get(struct ziface_struct *ifaces, int max_interfaces)
-{
-    int total, i, j;
-
-    total = _interfaces_get(ifaces, max_interfaces);
-    if (total <= 0) return total;
-
-    /* now we need to remove duplicates */
-    qsort(ifaces, total, sizeof(ifaces[0]), (int (*)(const void *, const void *))iface_comp);
-
-    for (i=1;i<total;) {
-        if (iface_comp(&ifaces[i-1], &ifaces[i]) == 0) {
-            for (j=i-1;j<total-1;j++) {
-                ifaces[j] = ifaces[j+1];
-            }
-            total--;
-        } else {
-            i++;
-        }
-    }
-
-    return total;
-}
-
-
 
 #else
 
@@ -172,10 +225,14 @@ int zifaces_get(struct ziface_struct *ifaces, int max_interfaces)
 #include <windows.h>
 #include <iphlpapi.h>
 
+#pragma warning(disable : 4996)
+
+
 typedef DWORD (CALLBACK *T_GetAdaptersInfo)(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen);
 
+
 /* GetAdaptersInfo is since Win2k */
-int zifaces_get(struct ziface_struct *ifaces, int max_interfaces){
+int _interfaces_get(struct ziface_struct *ifaces, int max_interfaces, int up_only/*under win32 only up are returned*/){
     PIP_ADAPTER_INFO pAdapterInfo;
 	PIP_ADAPTER_INFO pAdapter;
 	int i;
@@ -220,6 +277,11 @@ int zifaces_get(struct ziface_struct *ifaces, int max_interfaces){
 		pAdapter != NULL && i < max_interfaces; 
 		pAdapter = pAdapter->Next) {
 
+		if (up_only && z_strcasestr(pAdapter->Description, "VirtualBox") != NULL)
+		{
+			continue;
+		}
+
 		ifaces[i].ip.S_un.S_addr = inet_addr(pAdapter->IpAddressList.IpAddress.String);
 		if (ifaces[i].ip.S_un.S_addr == 0L) continue;
 		ifaces[i].netmask.S_un.S_addr = inet_addr(pAdapter->IpAddressList.IpMask.String);
@@ -231,21 +293,75 @@ int zifaces_get(struct ziface_struct *ifaces, int max_interfaces){
 			case MIB_IF_TYPE_PPP:        g_snprintf(ifaces[i].name, ZIFACE_NAME_SIZE, "ppp%d", i); break;
 			case MIB_IF_TYPE_LOOPBACK:   g_snprintf(ifaces[i].name, ZIFACE_NAME_SIZE, "lo"); break;
 			case MIB_IF_TYPE_SLIP:       g_snprintf(ifaces[i].name, ZIFACE_NAME_SIZE, "sl%d", i); break;
+			case IF_TYPE_IEEE80211:		 g_snprintf(ifaces[i].name, ZIFACE_NAME_SIZE, "wlan%d", i); break;
 			default: 			 	 	 g_snprintf(ifaces[i].name, ZIFACE_NAME_SIZE, "unk%d", i); break;
 		}
+		memcpy(ifaces[i].mac, pAdapter->Address, 6);
 		i++;
 	}
 
 	return i;
 }
 
+int ziface_wifi_stats(struct zwifi_stat *stat, char *devname, int clear){
+	static int i = 0;
+
+	stat->level_dbm = i - 100;
+	stat->quality_percent = i;
+
+	i += 10;
+	if (i > 100) i = 0;
+
+
+	return 0;
+}
+
 #endif
 
 
+static int iface_comp(struct ziface_struct *i1, struct ziface_struct *i2)
+{
+	int r;
+	r = strcmp(i1->name, i2->name);
+	if (r) return r;
+	r = ntohl(i1->ip.s_addr) - ntohl(i2->ip.s_addr);
+	if (r) return r;
+	r = ntohl(i1->netmask.s_addr) - ntohl(i2->netmask.s_addr);
+	return r;
+}
+
+/* this wrapper is used to remove duplicates from the interface list generated
+above */
+int zifaces_get(struct ziface_struct *ifaces, int max_interfaces, int up_only)
+{
+	int total, i, j;
+
+	total = _interfaces_get(ifaces, max_interfaces, up_only);
+	if (total <= 0) return total;
+
+	/* now we need to remove duplicates */
+	qsort(ifaces, total, sizeof(ifaces[0]), (int (*)(const void *, const void *))iface_comp);
+
+	for (i=1;i<total;) {
+		if (iface_comp(&ifaces[i-1], &ifaces[i]) == 0) {
+			for (j=i-1;j<total-1;j++) {
+				ifaces[j] = ifaces[j+1];
+			}
+			total--;
+		} else {
+			i++;
+		}
+	}
+
+	return total;
+}
+
+
+
+
 #ifdef AUTOCONF_TEST
 /* this is the autoconf driver to test get_interfaces() */
 
-#define MAX_INTERFACES 128
 
 int main()
 {
@@ -266,6 +382,10 @@ int main()
         b = ifaces[i].ip.s_addr | ~(ifaces[i].netmask.s_addr);
         iaddr.s_addr  = b;
         printf("BCAST=%-15s\n", inet_ntoa(iaddr));
+        printf("MAC=");
+        for (b = 0; b < 6; b++) printf("%02X ", ifaces[i].mac[b]);
+        printf("\n");
+        
     }
     return 0;
 }
@@ -279,7 +399,7 @@ int ziface_is_local(struct in_addr ia){
 
 	if ((ntohl(ia.s_addr) & 0xff000000) == 0x7f000000) return 1;
 
-	ret = zifaces_get(ifaces, ZMAX_INTERFACES);
+	ret = zifaces_get(ifaces, ZMAX_INTERFACES, 1);
 	for (i = 0; i < ret; i++){
 		unsigned long mynet = ifaces[i].ip.s_addr & ifaces[i].netmask.s_addr;
 		unsigned long peernet = ia.s_addr & ifaces[i].netmask.s_addr;
@@ -287,3 +407,28 @@ int ziface_is_local(struct in_addr ia){
 	}
 	return 0;
 }
+
+
+char *ziface_macid(const char *namestarts){
+    static char macid[32];
+    int total, i, up_only;
+
+    struct ziface_struct ifaces[MAX_INTERFACES];
+
+    for (up_only = 1; up_only >= 0; up_only--){
+		total = zifaces_get(ifaces, MAX_INTERFACES, up_only);
+        if (total <= 0) return NULL;
+
+        for (i=0;i<total;i++) {
+
+            if (namestarts != NULL){
+                char *c = strstr(ifaces[i].name, namestarts);
+                if (c == NULL) continue;
+            }
+            
+            g_snprintf(macid, 32, "%02X%02X", ifaces[i].mac[4], ifaces[i].mac[5]);
+            return macid;
+        }
+    }											 
+    return NULL;
+}
