@@ -1,6 +1,6 @@
 /*
     PNG functions
-    Copyright (C) 2002-2011 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2020 Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of tuxpaint http://www.newbreedsoftware.com/tuxpaint/
     and authors of libsdl-image
 
@@ -165,6 +165,9 @@ static void png_read_data(png_structp ctx, png_bytep area, png_size_t size)
 	SDL_RWread(src, area, size, 1);
 }
 
+void zpng_error_fce(png_structp png, png_const_charp ch){
+}
+
 static SDL_Surface *IMG_LoadPNG_RW(SDL_RWops *src)
 {
 	SDL_Surface *volatile surface;
@@ -194,16 +197,16 @@ static SDL_Surface *IMG_LoadPNG_RW(SDL_RWops *src)
 
 	/* Create the PNG loading context structure */
 	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
-					  NULL,NULL,NULL);
+		NULL, zpng_error_fce, NULL);
 	if (png_ptr == NULL){
-		error("Couldn't allocate memory for PNG file or incompatible PNG dll");
+		error("Couldn't allocate memory for PNG file or incompatible PNG dll (%s)\n", PNG_LIBPNG_VER_STRING);
 		goto done;
 	}
 
 	 /* Allocate/initialize the memory for image information.  REQUIRED. */
 	info_ptr = png_create_info_struct(png_ptr);
 	if (info_ptr == NULL) {
-		error("Couldn't create image information for PNG file");
+		error("Couldn't create image information for PNG file\n");
 		goto done;
 	}
 
@@ -212,7 +215,7 @@ static SDL_Surface *IMG_LoadPNG_RW(SDL_RWops *src)
 	 * set up your own error handlers in png_create_read_struct() earlier.
 	 */
 	if ( setjmp(png_jmpbuf(png_ptr)) ) {
-		error("Error reading the PNG file.");
+		error("Error reading the PNG file.\n");
 		goto done;
 	}
 
@@ -295,7 +298,7 @@ static SDL_Surface *IMG_LoadPNG_RW(SDL_RWops *src)
 			//bit_depth*info_ptr->channels, Rmask,Gmask,Bmask,Amask);
 			bit_depth*png_get_channels(png_ptr, info_ptr), Rmask,Gmask,Bmask,Amask);
 	if ( surface == NULL ) {
-		error("Out of memory");
+		error("Out of memory\n");
 		goto done;
 	}
 
@@ -314,7 +317,7 @@ static SDL_Surface *IMG_LoadPNG_RW(SDL_RWops *src)
 	/* Create the array of pointers to image data */
 	row_pointers = (png_bytep*) malloc(sizeof(png_bytep)*height);
 	if (row_pointers == NULL) {
-		error("Out of memory");
+		error("Out of memory\n");
 		SDL_FreeSurface(surface);
 		surface = NULL;
 		goto done;
@@ -399,6 +402,6 @@ SDL_Surface *zpng_create(const void *data, int len){
 
 #ifdef Z_HAVE_LIBPNG
 void zpng_get_version(GString *gs){
-	g_string_sprintfa(gs, "%d.%d.%d", PNG_LIBPNG_VER_MAJOR, PNG_LIBPNG_VER_MINOR, PNG_LIBPNG_VER_RELEASE);
+	g_string_append_printf(gs, "%d.%d.%d", PNG_LIBPNG_VER_MAJOR, PNG_LIBPNG_VER_MINOR, PNG_LIBPNG_VER_RELEASE);
 }
 #endif
