@@ -1,6 +1,6 @@
 /*
     zsdl.c - reusable SDL functions
-    Copyright (C) 2011 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2011-2018 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -31,6 +31,9 @@
 
 #include <math.h>
 
+#include <zdebug.h>
+#include <ziconv.h>
+
 #ifdef Z_HAVE_SDL
 #include <glib.h>
 //#include <SDL.h>
@@ -40,7 +43,6 @@
 //#endif
 
 
-#include <zdebug.h>
 #include <zpng.h>
 #include <zthread.h>
 
@@ -106,7 +108,7 @@ struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse){
 //    zsdl_font_dump("font13x24.png", "font13x24.inc");
 //    zsdl_font_dump_skip_red("font8x13red.png", "font8x13.inc");
 //    zsdl_font_dump("font8x13.png", "font8x13.inc");
-//    zsdl_font_dump("font18x32a.png", "font18x32.inc");
+    //zsdl_font_dump("font18x32apng", "font18x32.inc");
    return zsdl;
 }
 
@@ -114,6 +116,11 @@ struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse){
 void zsdl_free(void){
     if (!zsdl) return;
 	g_free(outline9x16);
+
+	if (zsdl->to8b_cd != NULL) iconv_close(zsdl->to8b_cd);
+	zsdl->to8b_cd = NULL;
+	g_string_free(zsdl->to8b_gs, TRUE);
+
     g_free(zsdl);
     zsdl = NULL;
 }
@@ -472,6 +479,15 @@ void zsdl_printf(SDL_Surface *surface, int x, int y, int color, int bgcolor, int
         va_start(l, m);
         s = g_strdup_vprintf(m, l);
         va_end(l);
+
+		if (flags & ZFONT_UTF8){
+			if (zsdl->to8b_cd == NULL) zsdl->to8b_cd = iconv_open("iso-8859-2//TRANSLIT", "utf-8");
+			if (zsdl->to8b_gs == NULL) zsdl->to8b_gs = g_string_sized_new(strlen(s));
+			g_string_truncate(zsdl->to8b_gs, 0);
+			ziconv(zsdl->to8b_cd, s, zsdl->to8b_gs);
+			g_free(s);
+			s = g_strdup(zsdl->to8b_gs->str);
+		}
     }
 
     if (flags & ZFONT_CENTERX) x -= (strlen(s) * zsdl->font_w / 2);
@@ -596,7 +612,21 @@ void zsdl_printf(SDL_Surface *surface, int x, int y, int color, int bgcolor, int
 
 	if (SDL_MUSTLOCK(surface)) SDL_UnlockSurface(surface);
 
-    if (flags & ZFONT_SYNC) SDL_UpdateRect(surface, xx, y, strlen(s) * zsdl->font_w, font_h);
+	if (flags & ZFONT_SYNC) {
+		int hh = font_h;
+		int ww = xx + strlen(s) * zsdl->font_w;
+		if (xx + ww >= surface->w) ww = surface->w - xx - 1;
+		if (y + hh >= surface->h) hh = surface->h - y - 1;
+		if (xx < 0) {
+			ww += xx - 1;
+			xx = 0;
+		}
+		if (y < 0) {
+			hh += y - 1;
+			y = 0;
+		}
+		SDL_UpdateRect(surface, xx, y, ww, font_h);
+	}
     if (!(flags & ZFONT_CHAR)) g_free(s);
 	if (flags & ZFONT_USERHFLG)	{
 		zsdl->font_w = old_font_w;
@@ -605,6 +635,19 @@ void zsdl_printf(SDL_Surface *surface, int x, int y, int color, int bgcolor, int
 }
 
 
+void zsdl_fit(int *flg, int width, int height, char *str){
+	int widths[] = { 32, 24, 16, 13, 0 };
+	int i, fw, fh;
+	
+	for (i = 0; widths[i] != 0; i++){
+		fh = widths[i];
+		if (fh > height) continue;
+		fw = zsdl_h2w(fh);
+		if (fw * (int)strlen(str) < width) break;
+	}
+	*flg |= ZFONT_USERH(fh);
+}
+
 
 void z_lineaa(SDL_Surface *surface, int x1, int y1, int x2, int y2, int color){
 /*void DrawWuLine (CDC *pDC, short X0, short Y0, short X1, short Y1,
@@ -915,6 +958,38 @@ void z_triangle(SDL_Surface *surface, int x1, int y1, int x2, int y2, int x3, in
     
 }
 
+void z_triangle_net(SDL_Surface *surface, int c, int n, ...){
+    va_list l;
+	int x1, y1, x2, y2, x3, y3;
+
+	if (n < 3) return;
+
+    va_start(l, n);
+
+	x1 = va_arg(l, int);
+	y1 = va_arg(l, int);
+	x2 = va_arg(l, int);
+	y2 = va_arg(l, int);
+	x3 = va_arg(l, int);
+	y3 = va_arg(l, int);
+	n-=3;
+
+	while(1){
+		z_triangle(surface, x1, y1, x2, y2, x3, y3, c);
+		if (n == 0) break;
+
+		n--;
+		x1 = x2;
+		y1 = y2;
+		x2 = x3;
+		y2 = y3;
+		x3 = va_arg(l, int);
+		y3 = va_arg(l, int);
+	}
+    va_end(l);
+}
+
+
 void z_circle(SDL_Surface *surface, int x, int y, int r, int c){
     int xx, yy, p;
 
@@ -1309,6 +1384,9 @@ int zsdl_sys_key_repeat(int delay, int rate){
 
 #endif
 
+#ifdef Z_ANDROID
+    void SDL_ANDROID_DisableScreensaver(void);
+#endif
 
 void z_disable_screensaver(void){
 #ifdef Z_MSC_MINGW
@@ -1413,7 +1491,19 @@ int zsdl_maximized(struct zzsdl *zsdl, int *w, int *h){
 #endif
 	
 }
-#endif
+
+int z_color_from_html(char *htmlcolor){
+	int r, g, b, n;
+	char *c = htmlcolor;
+	if (*c == '#') c++;
+
+	if (strlen(c) < 6) return -1;
+	n = sscanf(c, "%02x%02x%02x", &r, &g, &b);
+	if (n != 3) return -2;
+
+	return z_makecol(r, g, b);
+}
+
 
 int z_line_intersect(double x1, double y1, double x2, double y2, double x3, double y3, double x4, double y4, double *x, double *y){
 	double a1, b1, c1, a3, b3, c3, d, ix, iy;
@@ -1486,6 +1576,5 @@ int z_point_is_in_quadrangle(double xp, double yp, double x1, double y1, double
     return 1;
 }
 
-
-
+#endif
 
