@@ -1,6 +1,6 @@
 /*
     select.c - main application loop
-    Copyright (C) 2002-2012  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2020  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
     This program is free software; you can redistribute it and/or
@@ -19,6 +19,7 @@
 #include <ztime.h>
 
 #include <errno.h>
+#include <math.h>
 #include <stdarg.h>
 #include <string.h>
 #ifdef Z_HAVE_SYS_SELECT_H
@@ -147,13 +148,7 @@ struct zselect *zselect_init(void (*redraw)(void)){
 	zsel->fds3 = g_ptr_array_new();
 #endif
 
-#ifdef Z_UNIX
-    if (!signal_init){
-        signal_init = 1;
-        memset(signal_mask, 0, sizeof signal_mask);
-        memset(signal_handlers, 0, sizeof signal_handlers);
-    }
-#endif
+    zselect_signal_init();
     zsel->redraw = redraw;
 
     if (z_pipe(zsel->msgpipe)) zinternal("zselect_init: can't create msg pipe");
@@ -323,6 +318,48 @@ int zselect_timer_new_dbg(struct zselect *zsel, ztime t, void (*func)(void *), c
     return tm->id;
 }
 
+int zselect_timer_new_at(struct zselect *zsel, struct timeval *at, void (*func)(void *), void *arg)
+{
+    struct ztimer *tm, *tt;
+#ifdef Z_LEAK_DEBUG_LIST
+    tm = debug_g_malloc(file, line, sizeof(struct ztimer));
+    tm->fname = fname;
+#else            
+    tm = g_new(struct ztimer, 1);
+#endif        
+    if (!tm) return -1;
+    
+    struct timeval now;
+    gettimeofday(&now, NULL);
+    double d = z_difftimeval_double(at, &now);
+    ztime t = (ztime)round(d*1000.0);
+    tm->interval = t;
+    tm->func = func;
+	//tm->fname = fname;
+    tm->arg = arg;
+    tm->id = zsel->timer_id++;
+#if 0
+	dbg("before:\n");
+	foreach(tt, timers){
+		dbg("    id=%d inter=%d fce=%p\n",tt->id, tt->interval, tt->func);  
+	}
+#endif	
+	
+    foreach(tt, zsel->timers) if (tt->interval >= t) break;
+    add_at_pos(tt->prev, tm);
+	/*if (t != 50){
+    dbg("__installed ztimer %d %p t=%d\n",tm->id, func,t );
+	} */
+#if 0
+	dbg("after:\n");
+	foreach(tt, timers){
+		dbg("    id=%d inter=%d fce=%p\n",tt->id, tt->interval, tt->func);  
+	}
+	dbg("\n");
+#endif	
+    return tm->id;
+}
+
 #ifdef Z_LEAK_DEBUG_LIST
 void debug_kill_timer(char *file, int line, int id)
 #else    
@@ -376,36 +413,6 @@ void *zselect_get(struct zselect *zsel, int sock, int typ)
     return NULL;
 }
 
-void zselect_set_write(struct zselect *zsel, int fd, void (*write_func)(void *), void *arg){
-	struct zselect_fd *zfd = get_fd(zsel, fd);
-    
-    zfd->write_func = write_func;
-	zfd->write_fname = NULL;//write_fname;
-    zfd->arg = arg;
-    
-    if (write_func) FD_SET(fd, &zsel->w_write);
-    else {
-        FD_CLR(fd, &zsel->w_write);
-        FD_CLR(fd, &zsel->x_write);
-    }
-
-#ifdef Z_MSC_MINGW
-	if (!zfd->read_func && !write_func && !zfd->error_func){
-		g_ptr_array_remove_fast(zsel->fds3, zfd);
-	}
-	zsel->w_max = zsel->fds3->len;
-#else
-    if (zfd->read_func || write_func || zfd->error_func) {
-        if (fd >= zsel->w_max) zsel->w_max = fd + 1;
-    } else if (fd == zsel->w_max - 1) {
-        int i;
-        for (i = fd - 1; i >= 0; i--)
-            if (FD_ISSET(i, &zsel->w_read) || FD_ISSET(i, &zsel->w_write) ||
-                FD_ISSET(i, &zsel->w_error)) break;
-        zsel->w_max = i + 1;
-    }
-#endif
-}
 
 #ifdef Z_LEAK_DEBUG_LIST
 void debug_set_handlers(char *file, int line, int fd, 
@@ -472,6 +479,66 @@ void zselect_set_dbg(struct zselect *zsel, int fd, void (*read_func)(void *), ch
 }
 
 
+void zselect_set_read(struct zselect *zsel, int fd, void (*read_func)(void *), void *arg){
+	struct zselect_fd *zfd = get_fd(zsel, fd);
+    zfd->read_func = read_func;
+	zfd->arg = arg;
+   
+	if (read_func) FD_SET(fd, &zsel->w_read);
+    else {
+        FD_CLR(fd, &zsel->w_read);
+        FD_CLR(fd, &zsel->x_read);
+    }
+
+#ifdef Z_MSC_MINGW
+	if (!read_func && !zfd->write_func && !zfd->error_func){
+		g_ptr_array_remove_fast(zsel->fds3, zfd);
+	}
+	zsel->w_max = zsel->fds3->len;
+#else
+    if (read_func || zfd->write_func || zfd->error_func) {
+        if (fd >= zsel->w_max) zsel->w_max = fd + 1;
+    } else if (fd == zsel->w_max - 1) {
+        int i;
+        for (i = fd - 1; i >= 0; i--)
+            if (FD_ISSET(i, &zsel->w_read) || FD_ISSET(i, &zsel->w_write) ||
+                FD_ISSET(i, &zsel->w_error)) break;
+        zsel->w_max = i + 1;
+    }
+#endif
+
+}
+
+void zselect_set_write(struct zselect *zsel, int fd, void (*write_func)(void *), void *arg){
+	struct zselect_fd *zfd = get_fd(zsel, fd);
+    zfd->write_func = write_func;
+	zfd->arg = arg;
+   
+	if (write_func) FD_SET(fd, &zsel->w_write);
+    else {
+        FD_CLR(fd, &zsel->w_write);
+        FD_CLR(fd, &zsel->x_write);
+    }
+
+#ifdef Z_MSC_MINGW
+	if (!zfd->read_func && !write_func && !zfd->error_func){
+		g_ptr_array_remove_fast(zsel->fds3, zfd);
+	}
+	zsel->w_max = zsel->fds3->len;
+#else
+    if (zfd->read_func || write_func || zfd->error_func) {
+        if (fd >= zsel->w_max) zsel->w_max = fd + 1;
+    } else if (fd == zsel->w_max - 1) {
+        int i;
+        for (i = fd - 1; i >= 0; i--)
+            if (FD_ISSET(i, &zsel->w_read) || FD_ISSET(i, &zsel->w_write) ||
+                FD_ISSET(i, &zsel->w_error)) break;
+        zsel->w_max = i + 1;
+    }
+#endif
+
+}
+
 int critical_section = 0;
 
 void check_for_select_race(void);
@@ -491,6 +558,8 @@ void got_signal(int sig, siginfo_t *siginfo, void *ctx)
         sig_segv(NULL); 
         return;
     } */
+
+    //dbg("    handler[%d] = %p, critical = %d\n", sig, signal_handlers[sig].fn, signal_handlers[sig].critical);
     if (!signal_handlers[sig].fn) return;
     if (signal_handlers[sig].critical) {
         signal_handlers[sig].fn(signal_handlers[sig].arg, siginfo, ctx);
@@ -505,7 +574,7 @@ void zselect_signal_set(int sig, void (*fn)(void *, siginfo_t *, void *ctx), voi
 {
 //#ifdef Z_HAVE_SIGNAL_H  //under mingw is signal.h without SIGPIPE
     struct sigaction sa;
-//    dbg("zselect_signal_set(%d, %p, %p, %d)\n", sig, fn, arg, critical);
+    dbg("zselect_signal_set(%d, %p, %p, %d)\n", sig, fn, arg, critical);
     if (sig >= NUM_SIGNALS || sig < 0) {
         zinternal("bad signal number: %d", sig);
         return;
@@ -658,15 +727,15 @@ void zselect_loop(struct zselect *zsel)
                 struct timeval tv;
                 GString *gs = g_string_sized_new(1024);
 
-                g_string_sprintfa(gs, "ERROR: select failed: %d", err);
+                g_string_append_printf(gs, "ERROR: select failed: %d", err);
 				
                 FD_ZERO(&used);
                 g_string_append(gs, "\nR:");
-                for (i = 0; i < 256; i++) if (FD_ISSET(i, &zsel->x_read)) { g_string_sprintfa(gs, "%d,", i); FD_SET(i, &used); }
+                for (i = 0; i < 256; i++) if (FD_ISSET(i, &zsel->x_read)) { g_string_append_printf(gs, "%d,", i); FD_SET(i, &used); }
                 g_string_append(gs, "\nW:");
-                for (i = 0; i < 256; i++) if (FD_ISSET(i, &zsel->x_write)) { g_string_sprintfa(gs, "%d,", i); FD_SET(i, &used); }
+                for (i = 0; i < 256; i++) if (FD_ISSET(i, &zsel->x_write)) { g_string_append_printf(gs, "%d,", i); FD_SET(i, &used); }
                 g_string_append(gs, "\nE:");
-                for (i = 0; i < 256; i++) if (FD_ISSET(i, &zsel->x_error)) { g_string_sprintfa(gs, "%d,", i); FD_SET(i, &used); }
+                for (i = 0; i < 256; i++) if (FD_ISSET(i, &zsel->x_error)) { g_string_append_printf(gs, "%d,", i); FD_SET(i, &used); }
 
                 for (i = 0; i < 256; i++) {
                     if (!FD_ISSET(i, &used)) continue;
@@ -676,9 +745,9 @@ void zselect_loop(struct zselect *zsel)
                     tv.tv_usec=1;
                     if (select(i+1, &x, NULL, NULL, &tv)<0 && errno==EBADF)
 #ifdef Z_LEAK_DEBUG_LIST							
-                    g_string_sprintfa(gs, "\nfd %d failed(err=%d) at %s:%d\n", i, errno, zsel->fds[i].file, zsel->fds[i].line);
+                    g_string_append_printf(gs, "\nfd %d failed(err=%d) at %s:%d\n", i, errno, zsel->fds[i].file, zsel->fds[i].line);
 #else						
-                    g_string_sprintfa(gs, "\nfd %d failed(err=%d)\n ", i, errno);
+                    g_string_append_printf(gs, "\nfd %d failed(err=%d)\n ", i, errno);
 #endif							
                 }
                 
@@ -858,8 +927,26 @@ int zselect_profiling(struct zselect *zsel){
 	return zsel->profile;
 }
 
-/*void zselect_check_heap(void){
-#ifdef Z_MSC
-	_ASSERTE(_CrtCheckMemory());
+void zselect_abort_exception(void *arg){
+	zinternal("Socket exception handler called");
+
+}
+
+#ifdef Z_UNIX_ANDROID
+void zselect_sigterm(void *arg, siginfo_t *siginfo, void *ctx)
+{
+    struct zselect *zsel = (struct zselect *)arg;
+    zselect_terminate(zsel);
+}
 #endif
-}*/
+
+void zselect_signal_init(){
+#ifdef Z_UNIX
+    if (!signal_init){
+        signal_init = 1;
+        memset(signal_mask, 0, sizeof signal_mask);
+        memset(signal_handlers, 0, sizeof signal_handlers);
+    }
+#endif
+
+}
